//-------------------------------------------------------------------------------------------------------------------------------
{xml Headers and Footers}
function kml_Header1: String;
begin
  Result := '<?xml version="1.0"?>' + CRLF +
            '<kml xmlns="http://earth.google.com/kml/2.2">' + CRLF +
            '  <Document>' + CRLF +
            '    <name><![CDATA[PocketDrake]]></name>' + CRLF +
            '    <open>1</open><visibility>1</visibility>' + CRLF;
end;           
function kml_Header2(Name: String): String;
begin
  Result := '    <Placemark>' + CRLF +
            '      <Snippet></Snippet>' + CRLF +
            '      <Style><LineStyle><color>FFAB814B</color><width>4</width></LineStyle></Style>' + CRLF +
            '      <name><![CDATA[' + Name + ']]></name>' + CRLF +
            '      <description><![CDATA[Pocket Drake track]]></description>' + CRLF +
            '      <MultiGeometry>' + CRLF +
            '        <LineString>' + CRLF +
            '          <tessellate>1</tessellate>' + CRLF +
            '          <altitudeMode>clampToGround</altitudeMode>' + CRLF +
            '          <coordinates>' + CRLF;
end;           
function kml_Footer2(): String;
begin
  Result := '          </coordinates>' + CRLF +
            '        </LineString>' + CRLF +
            '      </MultiGeometry>' + CRLF +
            '    </Placemark>' + CRLF;
end;              
function kml_Footer1(): String;
begin
  Result := '  </Document>' + CRLF +
            '</kml>';
end;              
function plt_Header(Name: String): String;
begin
  Result := 'OziExplorer Track Point File Version 2.0' + CRLF +          //Line 1 : File type and version information
            'WGS 84' + CRLF +                                            //Line 2 : Geodetic Datum used for the Lat/Lon positions for each trackpoint
            'Altitude is in Feet' + CRLF +                               //Line 3 : "Altitude is in feet" - just a reminder that the altitude is always stored in feet
            'Reserved' + CRLF +                                          //Line 4 : Reserved for future use
            '0,2,255,' + Name + ',1' + CRLF +                            //Line 5 : multiple fields as below
                                                                                   {Field 1 : always zero (0)
                                                                                    Field 2 : width of track plot line on screen - 1 or 2 are usually the best
                                                                                    Field 3 : track color (RGB)
                                                                                    Field 4 : track description (no commas allowed)
                                                                                    Field 5 : track skip value - reduces number of track points plotted, usually set to 1 
                                                                                    Field 6 : track type - 0 = normal , 10 = closed polygon , 20 = Alarm Zone
                                                                                    Field 7 : track fill style - 0 =bsSolid; 1 =bsClear; 2 =bsBdiagonal; 3 =bsFdiagonal; 4 =bsCross; 
                                                                                                                 5 =bsDiagCross; 6 =bsHorizontal; 7 =bsVertical;
                                                                                    Field 8 : track fill color (RGB)  }
            '0' + CRLF;                                                  //Line 6 : Number of track points in the track, not used, the number of points is determined when reading the points file
end;           
function plt_Footer(): String;
begin
  Result := '';
end;              
function gpx_Header(Name: String): String;
begin
  Result := '<?xml version="1.0" encoding="UTF-16" standalone="no" ?>' + CRLF +
            '<gpx xmlns="http://www.topografix.com/GPX/1/1" creator="PocketDrakeSync" version="1.1">' + CRLF +
            '  <trk>' + CRLF +
            '    <name><![CDATA[' + Name + ']]></name>' + CRLF +
            '    <trkseg>' + CRLF;
end;           
function gpx_Footer(): String;
begin
  Result := '    </trkseg>' + CRLF +
            '  </trk>' + CRLF +
            '</gpx>';
end;              
//===============================================================================================================================
//-------------------------------------------------------------------------------------------------------------------------------
{Track Form Active/Inactive}
procedure form_TrackInh(Active: Boolean);
begin
  TrkForm_btn_Combine       .Enabled :=  Active;
  TrkForm_btn_Divide        .Enabled :=  Active;
  TrkForm_btn_Rename        .Enabled :=  Active;
  TrkForm_btn_New           .Enabled :=  Active;
  TrkForm_btn_Refresh       .Enabled :=  Active;
  TrkForm_btn_Upload        .Enabled :=  Active;
  TrkForm_btn_Download      .Enabled :=  Active;
  TrkForm_btn_Preview       .Enabled :=  Active;
  TrkForm_btn_Save          .Enabled :=  Active;
  TrkForm_btn_Delete        .Enabled :=  Active;
  TrkForm_btn_Trip          .Enabled :=  Active;
  TrkForm_btn_Close         .Enabled :=  Active;
  TrkForm_lb_Track          .Enabled :=  Active;
  TrkForm_ed_TrkName        .Enabled :=  Active;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Make a legal filename}
function SystemName(file: String): String;
begin
  Result := RegexReplace('\\|/|\*|\?|:|"|>|<|\|', file, '_', False);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Float to formated String}
function FormateFloat(Value: Extended): String;
var str : string;
begin
  if Value < 100 then begin
    str := IntToStr(trunc(Value * 10));
    Insert('.', str, length(str));
    Result := str;         
  end
  else Result := IntToStr(trunc(Value));
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Make DateTime from Track record}
function MakeDT(Track: TSqliteTable): TDateTime;
var
  str : String;
begin
  str    := RegexExtract('\d{6}$', '000000' + Track.FieldAsString(Track.FieldIndex['time']));    // doplni na format hhmmss a orizne
  Result := IntToDate(Track.FieldAsString(Track.FieldIndex['date'])) + EncodeTime(StrToInt(copy(str,1,2)),StrToInt(copy(str,3,2)),StrToInt(copy(str,5,2)),0);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Get Track ID from string}
function GetTrackID(str: String): String;
begin
  Result := RegexExtract('\d+$', str);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Get Track Name from string}
function GetTrackName(str: String): String;
begin
  Result := RegexExtract('^[^'+TAB+']*', str);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Create fix length string}
function FixLen(str: String; len: Integer): String;
var
  i : Integer;
begin
  if Length(str) > len then SetLength(str, len);
  for i := Length(str) to (len + 1) do begin
    str := str + TAB;
  end;
  Result := str;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Track list selection restore}
procedure TrackSelRest(ListID : TStringList);
var  
  i,j : Integer;
begin
  with TrkForm_lb_Track do begin
    for i := (Items.Count - 1) downto 0 do begin
      for j := (ListID.Count - 1) downto 0 do if GetTrackID(Items[i]) = ListID[j] then Selected[i] := True;   //v novem seznamu oznaci vznikle spojene stopy
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Arrange a ListBox}
procedure form_TrkArrName(Sender: TObject);
begin
//  TListBox(Sender).Sorted := True;
  TrkForm_lb_Track.Sorted := True;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Procedury formulare - zobrazeni zakl. info o stope}
procedure form_TrkInfo(Sender: TObject);
begin
  with TrkForm_lb_Track do begin
    TrkForm_ed_TrkName.Text := GetTrackName(Items[ItemIndex]);
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Read Track up}
procedure form_TrkRead(Sender: TObject);
var  
  Track_db            : TSqliteDatabase; // databaze s tabulkou Stopy 
  Track               : TSqliteTable;    // tabulka bodu Stopy
  str                 : String;
begin
  form_TrackInh(False);
  Track_db := TSqliteDatabase.Create(MakroPath + 'Data\$pocketdrake.db3');
  try
    with TrkForm_lb_Track do begin
      Sorted := False;
      Items.Clear;
      if SQLTableExists(Track_db, 'track_log') then begin
        case TBitBtn(Sender).Name of       // nacte tabulku stop dle zvoleneho razeni
          'btn_ArrDate_Dwn': Track := Track_db.GetTable('SELECT * FROM track_log WHERE lat="0" AND lon="0" ORDER BY date    ASC,  time ASC' , false);
          'btn_ArrID_Dwn'  : Track := Track_db.GetTable('SELECT * FROM track_log WHERE lat="0" AND lon="0" ORDER BY trackid ASC'            , false);
          'btn_ArrID_Up'   : Track := Track_db.GetTable('SELECT * FROM track_log WHERE lat="0" AND lon="0" ORDER BY trackid DESC'           , false);
          else               Track := Track_db.GetTable('SELECT * FROM track_log WHERE lat="0" AND lon="0" ORDER BY date    DESC, time DESC', false);
        end;
        if not SQLTableExists(Track_db, 'track_name') then Track_db.ExecSQL('CREATE TABLE track_name (trackid INTEGER PRIMARY KEY NOT NULL UNIQUE,name TEXT)');
        try
          while not Track.EOF do begin
            str := UtfToAnsi(Track_db.GetTableString('SELECT name FROM track_name WHERE trackid = ' + Track.FieldAsString(Track.FieldIndex['trackid'])));  //nazev stopy
            if str = '' then str := 'track_' + Track.FieldAsString(Track.FieldIndex['date']) + '_' + RegexExtract('\d{6}$', '000000' + Track.FieldAsString(Track.FieldIndex['time']));   //pokud neni nazev, vyrobit z datumu a casu
            str := FixLen(str,TrkForm_ed_TrkName.MaxLength+2) + FormatDateTime('dd.mm.yyyy hh:mm',  MakeDT(Track));
            Items.Add( str + TAB + TAB + TAB + FixLen(IntToStr(Track_db.GetTableValue('SELECT count(*) FROM track_log WHERE trackid="' + Track.FieldAsString(Track.FieldIndex['trackid']) + '"')),7) + TAB + Track.FieldAsString(Track.FieldIndex['trackid']));
            Track.next;                   // skok na dalsi
          end;
          if str = '' then Items.Add(_('- no track in table -'));
        except
          ShowErrorMessage(ExceptionToString(ExceptionType, ExceptionParam));
        finally
          Track.free;
        end;
      end
      else Items.Add('- track table not found -');
    end;
  except
    ShowErrorMessage(ExceptionToString(ExceptionType, ExceptionParam));
  finally
    Track_db.free;
  end;  
  form_TrackInh(True);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Divide Track}
procedure TrkDivide;
var  
  Track_db                 : TSqliteDatabase; // databaze s tabulkou Stopy 
  Track                    : TSqliteTable;    // tabulka bodu Stopy
  Track_num                : Integer;         // pocet zaznamu trasy v tabulce
  SelItems                 : TStringList;     // ktere stopy v seznamu maji byt na konci oznaceny
  dist, lat_last, lon_last : extended;
  dt, dt_last              : TDateTime;
  id                       : String;
  id_new                   : Integer;
  cond2, cond3             : boolean;
  i,j,k,m                  : Integer;
begin
  form_TrackInh(False);
  Track_db := TSqliteDatabase.Create(MakroPath + 'Data\$pocketdrake.db3');
  try
    with TrkForm_lb_Track do begin
      SelItems:=TStringList.Create;
      m := SelCount;  //pocet oznacenych stop v seznamu
      for i := (Items.Count - 1) downto 0 do begin 
        if Selected[i] then begin
          k := k + 1; j := 0; id_new := 0;
          TrkForm_pb_Track.Position := k * 100 / m;
          id := GetTrackID(Items[i]);
          SelItems.Add(id);    //prida puvodni oznacenou trasu
          if id <> '' then begin
            if TrkDivideForm_cb_HdopToTrimEnbl.Checked then begin
              Track_db.ExecSQL('DELETE FROM track_log WHERE trackid = ' + id + ' AND hdop > "' + TrkDivideForm_ed_HdopToTrim.Text + '"');   //nejprve odmazne zaznamy s velkym HDOP (nepresne vzorky)
            end;
            Track_num := Track_db.GetTableValue('SELECT count(*) FROM track_log WHERE trackid = ' + id + ' AND lat <> 0 AND lon <> 0');
            Track     := Track_db.GetTable     ('SELECT *        FROM track_log WHERE trackid = ' + id + ' AND lat <> 0 AND lon <> 0 ORDER BY date ASC, time ASC', false);
            try
              dt_last  := MakeDT(Track);
              lat_last := StrToFl(Track.FieldAsString(Track.FieldIndex['lat']));
              lon_last := StrToFl(Track.FieldAsString(Track.FieldIndex['lon']));
              Track_db.ExecSQL('BEGIN TRANSACTION');
              while not Track.EOF do begin
                if GB_Test then Break;  //preruseno uzivatelem
                j := j + 1;
                TrkForm_pb_Track2.Position := j * 100 / Track_num;
                dt   := MakeDT(Track)
                dist := PointDistance(lat_last, lon_last, StrToFl(Track.FieldAsString(Track.FieldIndex['lat'])), StrToFl(Track.FieldAsString(Track.FieldIndex['lon'])));
                cond2 := (abs(dt - dt_last) < TrkDivideForm_dt_TimeToTrim.Time) or not TrkDivideForm_cb_TimeToTrimEnbl.Checked;                  // deleni dle casovych rozestupu mezi dvema body
                cond3 := (dist < StrToFl(TrkDivideForm_ed_DistToTrim.Text))     or not TrkDivideForm_cb_TrackToTrimEnbl.Checked;                 // deleni dle vzdalenosti mezi dvema body
                if cond2 and cond3 then begin
                  if  id_new <> 0 then Track_db.ExecSQL('UPDATE track_log SET trackid = ' + IntToStr(id_new) + ' WHERE key = ' + Track.FieldAsString(Track.FieldIndex['key']));  //oznaci vzorky novym trackid    
                end
                else begin
                  Track_db.ExecSQL('COMMIT');
                  id_new := 0;
                  repeat         // hleda prvni volny trackid
                    id_new := id_new + 1; 
                  until Track_db.GetTableValue('SELECT count(*) FROM track_log WHERE trackid = ' + IntToStr(id_new)) = 0;
                  //zalozi nulovou znacku
                  Track_db.ExecSQL('INSERT INTO track_log (trackid,date,time,lat,lon,alt,hdop) VALUES(' + IntToStr(id_new) + ',' + Track.FieldAsString(Track.FieldIndex['date']) + ',' + Track.FieldAsString(Track.FieldIndex['time']) + ',0,0,0,0)');      
                  Track_db.ExecSQL('UPDATE track_log SET trackid = ' + IntToStr(id_new) + ' WHERE key = ' + Track.FieldAsString(Track.FieldIndex['key']));     
                  SelItems.Add(IntToStr(id_new));   //prida novou stopu do seznamu oznacenych radku
                  Track_db.ExecSQL('BEGIN TRANSACTION');
                end;
                dt_last  := dt;
                lat_last := StrToFl(Track.FieldAsString(Track.FieldIndex['lat']));
                lon_last := StrToFl(Track.FieldAsString(Track.FieldIndex['lon']));
                Track.next;                   // skok na dalsi
              end;
              Track_db.ExecSQL('COMMIT');
            finally
              Track.free;
            end;
          end;
        end;
      end;
    end;
  finally
    Track_db.free;
  end;  
  form_TrkRead(TrkForm_btn_Refresh);       //nacte aktualizovany seznam stop
  TrackSelRest(SelItems);                  //obnovi oznaceni polozek seznamu stop
  TrkForm_pb_Track.Position := 0;
  TrkForm_pb_Track2.Position := 0;
  form_TrackInh(True);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Combine Tracks}
procedure form_TrkCombine(Sender: TObject);
var
  Track_db     : TSqliteDatabase; // databaze s tabulkou Stopy 
  SelItems     : TStringList;     // ktere stopy v seznamu maji byt na konci oznaceny
  id, id_first : String;
  i, j         : Integer;
begin
  with TrkForm_lb_Track do begin
    if (SelCount > 0) then begin     // pokud je vybrana nejaka stopa
      form_TrackInh(False);
      SelItems:=TStringList.Create;
      Track_db := TSqliteDatabase.Create(MakroPath + 'Data\$pocketdrake.db3');
      try
        for i := (Items.Count - 1) downto 0 do begin 
          TrkForm_pb_Track.Position := (Items.Count -i) * 100 / Items.Count;
          if Selected[i] then begin
            id := GetTrackID(Items[i]);
            if id <> '' then begin
              if j > 0 then begin
                Track_db.ExecSQL('DELETE FROM track_log  WHERE trackid = "' + id + '" AND lat = 0 AND lon = 0');
                Track_db.ExecSQL('DELETE FROM track_name WHERE trackid = "' + id + '"');
                Track_db.ExecSQL('UPDATE track_log SET trackid = ' + id_first + ' WHERE trackid = ' + id);      
              end
              else begin 
                id_first := id;      //timto ID se oznaci cela spojovana stopa
                SelItems.Add(id);    //prida spojenou stopu mezi oznacene v seznamu
              end;
              j := j + 1;
            end;
          end;
        end;
      finally
        Track_db.free;
      end;
      form_TrkRead(TrkForm_btn_Refresh);      //nacte aktualizovany seznam stop
      TrackSelRest(SelItems); //obnovi oznaceni polozek seznamu stop
      TrkForm_pb_Track.Position := 0;
      form_TrackInh(True);
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Save Track}
function TrkSave(NameOfFile: String; SingleFile: Boolean): String;
var
  Track_db  : TSqliteDatabase; // databaze s tabulkou Stopy 
  Track     : TSqliteTable;    // tabulka bodu Stopy
  Track_num : Integer;         // pocet zaznamu trasy v tabulce
  xml       : String;
  FileName  : String;
  FileExt   : String;
  FilePath  : String;
  id, str   : String;
  i,j,k,m   : Integer;
begin
  with TrkForm_lb_Track do begin
      Track_db := TSqliteDatabase.Create(MakroPath + 'Data\$pocketdrake.db3');
      try
        FileExt    := RegexExtract('\.\w{3}$', NameOfFile);
        FilePath   := ReplaceString(NameOfFile, '\' + RegexExtract('[^\\]+\.\w{3}$', NameOfFile), '');
        m := SelCount;  //pocet oznacenych stop v seznamu
        for i := (Items.Count - 1) downto 0 do begin 
          if Selected[i] then begin
            TrkForm_pb_Track.Position := (k + 1) * 100 / m;
            id := GetTrackID(Items[i]);
            if id <> '' then begin
              k := k + 1;
              //pokud je vybrano vice stop, pak nazvy souboru dle stop
              if m > 1 then FileName := SystemName(GetTrackName(Items[i]))   //nahrazeni nepovolenych (mimo uvedenych) znaku
                       else FileName := ReplaceString(RegexExtract('[^\\]+\.\w{3}$', NameOfFile), FileExt, '');
              if not SingleFile or (k = 1) then NameOfFile := FilePath + '\' + FileName + FileExt;
              Track_num  := Track_db.GetTableValue('SELECT count(*) FROM track_log  WHERE trackid = "' + id + '" AND lat > 0 AND lon > 0');
              Track      := Track_db.GetTable     ('SELECT *        FROM track_log  WHERE trackid = "' + id + '" AND lat > 0 AND lon > 0 ORDER BY date ASC, time ASC', false);
              try
                case FileExt of 
                  '.kml': begin
                            if not SingleFile or (k = 1) then xml := kml_Header1;
                            xml := xml + kml_Header2(FileName);
                          end;
                  '.plt': xml := plt_Header(FileName);
                  '.gpx': xml := gpx_Header(FileName);
                end;
                j := 0;
                while not Track.EOF do begin
                  if GB_Test then Break;  //preruseno uzivatelem
                  j := j + 1;
                  TrkForm_pb_Track2.Position := j * 100 / Track_num;
                  case FileExt of 
                    '.kml': xml := xml + '            ' + Track.FieldAsString(Track.FieldIndex['lon']) + ',' + Track.FieldAsString(Track.FieldIndex['lat']) + ',' + Track.FieldAsString(Track.FieldIndex['alt']) + CRLF;                      //lon, lat, alt
                    '.plt': xml := xml + Track.FieldAsString(Track.FieldIndex['lat']) + ',' + Track.FieldAsString(Track.FieldIndex['lon']) + ',0,' + FloatToStr(StrToFloat(Track.FieldAsString(Track.FieldIndex['alt'])) * 3.28084) + ',' +   //lat, lon, code, alt(in feet)
                                         FloatToStr(IntToDate(Track.FieldAsString(Track.FieldIndex['date'])) + (StrToFloat(Track.FieldAsString(Track.FieldIndex['time'])) / 235959)) + ',,' + CRLF;                       //DateTime, [StringDate], [StringTime]
                    '.gpx': begin
                             str := RegexExtract('\d{6}$', '000000' + Track.FieldAsString(Track.FieldIndex['time']));
                             insert(':',str,3); insert(':',str,6);
                             str := Track.FieldAsString(Track.FieldIndex['date']) + 'T' + str + 'Z';
                             insert('-',str,5); insert('-',str,8);
                             xml := xml + '     <trkpt lat="' + Track.FieldAsString(Track.FieldIndex['lat']) + '" lon="' + Track.FieldAsString(Track.FieldIndex['lon']) + '">' +
                                               '<ele>' + Track.FieldAsString(Track.FieldIndex['alt']) + '</ele>' + 
                                               '<time>' + str + '</time>' +
                                               '<hdop>' + Track.FieldAsString(Track.FieldIndex['hdop']) + '</hdop>' +
                                               '</trkpt>' + CRLF;
                           end;
                  end;
                  Track.next;
                end;
                case FileExt of 
                  '.kml': begin
                            xml := xml + kml_Footer2;
                            if not SingleFile then xml := xml + kml_Footer1;
                          end;
                  '.plt': xml := xml + plt_Footer;
                  '.gpx': xml := xml + gpx_Footer;
                end;
                if not SingleFile then begin
                  MakeDirPC(FilePath);
                  StringToFile(xml, NameOfFile);
                end;
              finally
                Track.free;
              end;
            end;
          end;
        end;
        if xml <> '' then begin
          Result := NameOfFile;
          if SingleFile then begin
            MakeDirPC(FilePath);
            xml := xml + kml_Footer1;
            StringToFile(xml, NameOfFile);
          end;
        end;
      finally
        Track_db.free;
      end;
    end; 
    TrkForm_pb_Track.Position := 0;
    TrkForm_pb_Track2.Position := 0;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Preview Track by a kml asociated aplication}
procedure form_TrkView(Sender: TObject);
var
  file : String;
begin
  with TrkForm_lb_Track do begin
    if (SelCount > 0) then begin     // pokud je vybrana nejaka stopa
      form_TrackInh(False);
      file := TrkSave(MakroPath + 'Data\Preview.kml', True);
      if file <> '' then RunShell(file);
      form_TrackInh(True);
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Save Track to File}
procedure form_TrkSave(Sender: TObject);
var
  file : String;
begin
  with TrkForm_lb_Track do begin
    if (SelCount > 0) then begin     // pokud je vybrana nejaka stopa
      form_TrackInh(False);
      file := SystemName(GetTrackName(Items[ItemIndex])) + '.kml';  
      if SaveDlg(_('Save PocketDrake track'), SmartPath(TrackPathPC), '', _('GoogleEarth ') + '(*.kml)|*.kml|' + 
                                                                          _('OziExplorer ') + '(*.plt)|*.plt|' +
                                                                          _('GPX files ')   + '(*.gpx)|*.gpx', file) then begin
        if not RegexFind('((\.kml$)|(\.plt$))|(\.gpx$)', file) then file := file + '.kml';                                                                      
        TrkSave(file, False);
      end;
      form_TrackInh(True);
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Delete Track}
procedure form_TrkDelete(Sender: TObject);
var
  Track_db  : TSqliteDatabase; // databaze s tabulkou Stopy 
  id        : String;
  i         : Integer;
begin
  with TrkForm_lb_Track do begin
    if (SelCount > 0) then begin     // pokud je vybrana nejaka stopa
      form_TrackInh(False);
      Track_db := TSqliteDatabase.Create(MakroPath + 'Data\$pocketdrake.db3');
      try
        for i := (Items.Count - 1) downto 0 do begin 
          TrkForm_pb_Track.Position := (Items.Count -i) * 100 / Items.Count;
          if Selected[i] then begin
            id := GetTrackID(Items[i]);
            if id <> '' then begin
              Track_db.ExecSQL('DELETE FROM track_log  WHERE trackid = "' + id + '"');
              Track_db.ExecSQL('DELETE FROM track_name WHERE trackid = "' + id + '"');
              Items.Delete(i);
            end; 
          end;
        end;
        TrkForm_pb_Track.Position := 99;
        Track_db.ExecSQL('Vacuum');
      finally
        Track_db.free;
      end;
      TrkForm_pb_Track.Position := 0;
      form_TrackInh(True);
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Rename Track}
procedure form_TrkRename(Sender: TObject);
var
  Track_db    : TSqliteDatabase; // databaze s tabulkou Stopy 
  ii          : Integer;
begin
  with TrkForm_lb_Track do begin
    if (SelCount > 0) and (GetTrackID(Items[ItemIndex]) <> '') then begin       // pokud je vybrana nejaka stopa a je nactene ID
      Track_db := TSqliteDatabase.Create(MakroPath + 'Data\$pocketdrake.db3');
      try
        Track_db.ExecSQL('INSERT OR REPLACE INTO track_name (trackid, name) VALUES (' + GetTrackID(Items[ItemIndex]) + ', "' + AnsiToUtf(TrkForm_ed_TrkName.Text) + '")')
      finally
        Track_db.free;
      end;
      ii := ItemIndex;
      form_TrkRead(TrkForm_btn_Refresh);    //nacte aktualizovany seznam stop
      Selected[ii] := True; //oznaci puvodni radek
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Add a new Track}
procedure form_TrkNew(Sender: TObject);
var
  Track_db    : TSqliteDatabase; // databaze s tabulkou Stopy 
  id_new      : Integer;
begin
  Track_db := TSqliteDatabase.Create(MakroPath + 'Data\$pocketdrake.db3');
  try
    repeat         // hleda prvni volny trackid
      id_new := id_new + 1; 
    until Track_db.GetTableValue('SELECT count(*) FROM track_log WHERE trackid = ' + IntToStr(id_new)) = 0;
    //zalozi nulovou znacku
    Track_db.ExecSQL('INSERT INTO track_log (trackid,date,time,lat,lon,alt,hdop) VALUES(' + IntToStr(id_new) + ',' + FormatDateTime('yyyymmdd,hhmmss', now) + ',0,0,0,0)');      
  finally
    Track_db.free;
  end;
  form_TrkRead(TrkForm_btn_Refresh);    //nacte aktualizovany seznam stop
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Procedury formulare - zobrazeni Vyletu}
procedure form_TrkTrip(Sender: TObject);
var
  Track_db              : TSqliteDatabase; // databaze s tabulkou Stopy 
  Track                 : TSqliteTable;    // tabulka bodu Stopy
  Track_num             : Integer;         // pocet zaznamu trasy v tabulce
  id                    : String;
  lat_last, lon_last    : extended;
  TripLen, dist         : extended;
  alt_min, alt_max      : extended;
  dt, dt_first, dt_last : TDateTime;
  i,j,k,m               : Integer; 
begin
  with TrkForm_lb_Track do begin
    if (SelCount > 0) then begin     // pokud je vybrana nejaka stopa
      form_TrackInh(False);
      Track_db := TSqliteDatabase.Create(MakroPath + 'Data\$pocketdrake.db3');
      try
        k := 0; alt_min := 9999;  //9.9999 km
        m := SelCount;  //pocet oznacenych stop v seznamu
        for i := (Items.Count - 1) downto 0 do begin 
          if Selected[i] then begin
            k := k + 1;
            TrkForm_pb_Track.Position := k * 100 / m;
            id := GetTrackID(Items[i]);
            if id <> '' then begin
              Track_num  := Track_db.GetTableValue('SELECT count(*) FROM track_log  WHERE trackid = "' + id + '" AND lat > 0 AND lon > 0');
              Track      := Track_db.GetTable     ('SELECT *        FROM track_log  WHERE trackid = "' + id + '" AND lat > 0 AND lon > 0 ORDER BY date ASC, time ASC', false);
              try
                lat_last := StrToFl(Track.FieldAsString(Track.FieldIndex['lat']));
                lon_last := StrToFl(Track.FieldAsString(Track.FieldIndex['lon']));
                dt_first := MakeDT(Track);
                j := 0;
                while not Track.EOF do begin
                  j := j + 1;
                  TrkForm_pb_Track2.Position := j * 100 / Track_num;
                  //delka
                  dist    := PointDistance(lat_last, lon_last, StrToFl(Track.FieldAsString(Track.FieldIndex['lat'])), StrToFl(Track.FieldAsString(Track.FieldIndex['lon'])));
                  TripLen := TripLen + dist;
                  //cas
                  dt_last := MakeDT(Track);
                  //vyska
                  if (alt_min > StrToFl(Track.FieldAsString(Track.FieldIndex['alt']))) then alt_min := StrToFl(Track.FieldAsString(Track.FieldIndex['alt']));
                  if (alt_max < StrToFl(Track.FieldAsString(Track.FieldIndex['alt']))) then alt_max := StrToFl(Track.FieldAsString(Track.FieldIndex['alt']));
                  //-----
                  lat_last := StrToFl(Track.FieldAsString(Track.FieldIndex['lat']));
                  lon_last := StrToFl(Track.FieldAsString(Track.FieldIndex['lon']));
                  Track.next;
                end;
              finally
                Track.free;
              end;
              dt := dt + (dt_last - dt_first);
            end;
          end;
        end;
        TrkForm_ed_TripLenght.Text := FormateFloat(TripLen);
        TrkForm_ed_TripTime.Text   := IntToStr(trunc(dt)) + 'd  ' + FormatDateTime('hh:mm:ss', dt);
        TrkForm_ed_TripSpeed.Text  := FormateFloat(TripLen /(dt * 24));
        if (alt_min = 9999) then alt_min := 0;
        TrkForm_ed_TripAltMin.Text := IntToStr(trunc(alt_min));
        TrkForm_ed_TripAltMax.Text := IntToStr(trunc(alt_max));
      finally
        Track_db.free;
      end;
      TrkForm_pb_Track.Position := 0;
      TrkForm_pb_Track2.Position := 0;
      form_TrackInh(True);
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Procedury formulare - aktualizace stop z databze PD}
procedure form_TrkUpload(Sender: TObject);
begin
  form_TrackInh(False);
  TrkForm_pb_Track.Position := 10;
  // Nacteni nastaveni PocketDrake z registru PDA
  ReadRegPDA(True, PDSForm_rb_ActiveSync.Checked);
  TrkForm_pb_Track.Position := 20;
  if PDCard then begin
    TrkForm_pb_Track.Position := 30;
    // testuje zda neni v PDA spusten PocketDrake
    PD_Was_Run := Is_PD_Run;
    TrkForm_pb_Track.Position := 40;
    GB_Caption(_('Track from PocketDrake is loaded'));
    LogWork(_('* Track from PocketDrake is loaded'));
    if Is_PD_Run then PD_Quit;
    TrkForm_pb_Track.Position := 50;
    GB_Kind(_('- copying: $pocketdrake.db3...'));
    if PDSForm_rb_CardReader.Checked then CopyFile(ToCardPath(MapPath + '\$pocketdrake.db3'), MakroPath + 'Data\$pocketdrake.db3')
                                     else RunProc('"' + MakroPath + 'rapicopy.exe" -r "' + MapPath + '\$pocketdrake.db3' + '" "' + MakroPath + 'Data\$pocketdrake.db3' + '"', 'Copy_dbTrack_from_PDA', SetForm_cb_CmdWriteToFile.Checked); 
    TrkForm_pb_Track.Position := 90;
    LogWork(_('- track database from PDA to PC transfered'));
    // Spusteni PocketDrake
    if PD_Was_Run and not Is_PD_Run then PD_Run;
    TrkForm_pb_Track.Position := 100;
    form_TrkRead(TrkForm_btn_Refresh);    //nacte aktualizovany seznam stop
  end;
  form_TrackInh(True);
  GB_Clean;
  TrkForm_pb_Track.Position := 0;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Procedury formulare - download databze stop do PD}
procedure form_TrkDownload(Sender: TObject);
var
  Track_db  : TSqliteDatabase; // databaze s tabulkou Stopy 
begin
  form_TrackInh(False);
  TrkForm_pb_Track.Position := 10;
  if PDCard then begin
    // VACUUM na databazi
    Track_db := TSqliteDatabase.Create(MakroPath + 'Data\$pocketdrake.db3');
    try
      Track_db.ExecSQL('Vacuum');
    finally
      Track_db.free;
    end;
    TrkForm_pb_Track.Position := 20;
    // Nacteni nastaveni PocketDrake z registru PDA
    ReadRegPDA(True, PDSForm_rb_ActiveSync.Checked);
    TrkForm_pb_Track.Position := 30;
    TrkForm_pb_Track.Position := 40;
    // testuje zda neni v PDA spusten PocketDrake
    PD_Was_Run := Is_PD_Run;
    TrkForm_pb_Track.Position := 50;
    GB_Caption(_('Transfer track database to PocketDrake'));
    LogWork(_('* Transfer track database to PocketDrake'));
    if Is_PD_Run then PD_Quit;
    TrkForm_pb_Track.Position := 60;
    GB_Kind(_('- copying: $pocketdrake.db3...'));
    if PDSForm_rb_CardReader.Checked then CopyFile(MakroPath + 'Data\$pocketdrake.db3', ToCardPath(MapPath + '\$pocketdrake.db3'))
                                     else RunProc('"' + MakroPath + 'rapicopy.exe" "' + MakroPath + 'Data\$pocketdrake.db3' + '" "' + MapPath + '\$pocketdrake.db3' + '"', 'Copy_dbTrack_to_PDA', SetForm_cb_CmdWriteToFile.Checked);
    TrkForm_pb_Track.Position := 90;
    LogWork(_('- track database from PC to PDA transfered'));
    // Spusteni PocketDrake
    if PD_Was_Run and not Is_PD_Run then PD_Run;
  end;
  TrkForm_pb_Track.Position := 100;
  form_TrackInh(True);
  GB_Clean;
  TrkForm_pb_Track.Position := 0;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Zobrazi okno Tracks}
procedure form_ShowTrkForm(Sender: TObject);
begin
  form_TrkRead(TrkForm_btn_Refresh);    //nacte aktualizovany seznam stop
  GB_Caption(_('Tracks manager showed'));
  TrkForm.ShowModal;
end;
{Zobrazi okno TrkDivide}
procedure form_ShowTrkDivideForm(Sender: TObject);
begin
  if (TrkForm_lb_Track.SelCount > 0) then begin     // pokud je vybrana nejaka stopa
    ReadIniTrkForm('DEFAULT');
    if TrkDivideForm.ShowModal = 1 then begin
      TrkDivide;
      SaveIniTrkForm('DEFAULT');
    end;
  end;
end;
